@@ -21,17 +21,18 @@ class DBConfig(Configurable):
 
 
 # the database tables used in this example:
-tables = ('''seminars (
-    id varchar(4) primary key,
-    title varchar(64) unique not null,
-    cost money,
-    places_left smallint)''',
-'''attendees (
-    name varchar(64) not null,
-    seminar varchar(4),
-    paid boolean,
-    primary key(name, seminar),
-    foreign key (seminar) references seminars(id) on delete cascade)''')
+tables = (
+    '''seminars (
+        id varchar(4) primary key,
+        title varchar(64) unique not null,
+        cost money,
+        places_left smallint)''',
+    '''attendees (
+        name varchar(64) not null,
+        seminar varchar(4),
+        paid boolean,
+        primary key(name, seminar),
+        foreign key (seminar) references seminars(id) on delete cascade)''')
 
 
 class DBUtilsExample(ExamplePage):
@@ -44,15 +45,15 @@ class DBUtilsExample(ExamplePage):
     else:
         dbmod_name = 'Pooled'
     dbapi_name = config.pop('dbapi', 'pg')
-    if dbapi_name == 'pg': # use the PyGreSQL classic DB API
+    if dbapi_name == 'pg':  # use the PyGreSQL classic DB API
         dbmod_name += 'Pg'
-        if config.has_key('database'):
+        if 'database' in config:
             config['dbname'] = config['database']
             del config['database']
-        if config.has_key('password'):
+        if 'password' in config:
             config['passwd'] = config['password']
             del config['password']
-    else: # use a DB-API 2 compliant module
+    else:  # use a DB-API 2 compliant module
         dbmod_name += 'DB'
     dbapi = dbmod = dbclass = dbstatus = None
     try:
@@ -63,7 +64,7 @@ class DBUtilsExample(ExamplePage):
                 if dbapi_name != 'pg':
                     config['creator'] = dbapi
                 dbclass = getattr(dbmod, dbmod_name)(**config)
-            except dbapi.Error, error:
+            except dbapi.Error as error:
                 dbstatus = str(error)
             except Exception:
                 dbstatus = 'Could not connect to the database.'
@@ -75,16 +76,17 @@ class DBUtilsExample(ExamplePage):
     # Initialize the buttons
     _actions = []
     _buttons = []
-    for action in ('create tables',
-            'list seminars', 'list attendees',
+    for action in (
+            'create tables', 'list seminars', 'list attendees',
             'add seminar', 'add attendee'):
         value = action.capitalize()
         action = action.split()
         action[1] = action[1].capitalize()
         action = ''.join(action)
         _actions.append(action)
-        _buttons.append('<input name="_action_%s" '
-            'type="submit" value="%s">' % (action, value))
+        _buttons.append(
+            '<input name="_action_%s" type="submit" value="%s">'
+            % (action, value))
     _buttons = tuple(_buttons)
 
     def title(self):
@@ -106,8 +108,8 @@ class DBUtilsExample(ExamplePage):
         self._output.append(s)
 
     def outputMsg(self, msg, error=False):
-        self._output.append('<p style="color:%s">%s</p>'
-            % (error and 'red' or 'green', msg))
+        self._output.append(
+            '<p style="color:%s">%s</p>' % ('red' if error else 'green', msg))
 
     def connection(self, shareable=True):
         if self.dbstatus:
@@ -118,7 +120,7 @@ class DBUtilsExample(ExamplePage):
                     return self.dbclass.connection(shareable)
                 else:
                     return self.dbclass.connection()
-            except self.dbapi.Error, error:
+            except self.dbapi.Error as error:
                 error = str(error)
             except Exception:
                 error = 'Cannot connect to the database.'
@@ -138,8 +140,8 @@ class DBUtilsExample(ExamplePage):
         if not db:
             return
         for table in tables:
-            self._output.append('<p>Creating the following table:</p>'
-                '<pre>%s</pre>' % table)
+            self._output.append(
+                '<p>Creating the following table:</p><pre>%s</pre>' % table)
             ddl = 'create table ' + table
             try:
                 if self.dbapi_name == 'pg':
@@ -147,7 +149,7 @@ class DBUtilsExample(ExamplePage):
                 else:
                     db.cursor().execute(ddl)
                     db.commit()
-            except self.dbapi.Error, error:
+            except self.dbapi.Error as error:
                 if self.dbapi_name != 'pg':
                     db.rollback()
                 self.outputMsg(error, True)
@@ -158,7 +160,7 @@ class DBUtilsExample(ExamplePage):
     def listSeminars(self):
         id = self.request().field('id', None)
         if id:
-            if type(id) != type([]):
+            if not isinstance(id, list):
                 id = [id]
             cmd = ','.join(map(self.sqlEncode, id))
             cmd = 'delete from seminars where id in (%s)' % cmd
@@ -173,7 +175,7 @@ class DBUtilsExample(ExamplePage):
                 else:
                     db.cursor().execute(cmd)
                     db.commit()
-            except self.dbapi.Error, error:
+            except self.dbapi.Error as error:
                 try:
                     if self.dbapi_name == 'pg':
                         db.query('end')
@@ -188,8 +190,8 @@ class DBUtilsExample(ExamplePage):
         db = self.connection()
         if not db:
             return
-        query = ('select id, title, cost, places_left from seminars '
-            'order by title')
+        query = ('select id, title, cost, places_left'
+                 ' from seminars order by title')
         try:
             if self.dbapi_name == 'pg':
                 result = db.query(query).getresult()
@@ -198,7 +200,7 @@ class DBUtilsExample(ExamplePage):
                 cursor.execute(query)
                 result = cursor.fetchall()
                 cursor.close()
-        except self.dbapi.Error, error:
+        except self.dbapi.Error as error:
             self.outputMsg(error, True)
             return
         if not result:
@@ -223,21 +225,22 @@ class DBUtilsExample(ExamplePage):
     def listAttendees(self):
         id = self.request().field('id', None)
         if id:
-            if type(id) != type([]):
+            if not isinstance(id, list):
                 id = [id]
-            cmds = ['delete from attendees '
-                'where rpad(seminar,4)||name in (%s)'
+            cmds = [
+                'delete from attendees where rpad(seminar,4)||name in (%s)'
                 % ','.join(map(self.sqlEncode, id))]
             places = {}
             for i in id:
                 i = i[:4].rstrip()
-                if places.has_key(i):
+                if i in places:
                     places[i] += 1
                 else:
                     places[i] = 1
             for i, n in places.items():
-                cmds.append("update seminars set places_left=places_left+%d "
-                "where id=%s" % (n, self.sqlEncode(i)))
+                cmds.append(
+                    'update seminars set places_left=places_left+%d'
+                    ' where id=%s' % (n, self.sqlEncode(i)))
             db = self.dedicated_connection()
             if not db:
                 return
@@ -251,7 +254,7 @@ class DBUtilsExample(ExamplePage):
                     for cmd in cmds:
                         db.cursor().execute(cmd)
                     db.commit()
-            except self.dbapi.Error, error:
+            except self.dbapi.Error as error:
                 if self.dbapi_name == 'pg':
                     db.query('end')
                 else:
@@ -263,10 +266,9 @@ class DBUtilsExample(ExamplePage):
         db = self.connection()
         if not db:
             return
-        query = ('select a.name, s.id, s.title, a.paid '
-            ' from attendees a,seminars s'
-            ' where s.id=a.seminar'
-            ' order by a.name, s.title')
+        query = ('select a.name, s.id, s.title, a.paid'
+                 ' from attendees a,seminars s'
+                 ' where s.id=a.seminar order by a.name, s.title')
         try:
             if self.dbapi_name == 'pg':
                 result = db.query(query).getresult()
@@ -275,7 +277,7 @@ class DBUtilsExample(ExamplePage):
                 cursor.execute(query)
                 result = cursor.fetchall()
                 cursor.close()
-        except self.dbapi.Error, error:
+        except self.dbapi.Error as error:
             self.outputMsg(error, True)
             return
         if not result:
@@ -288,7 +290,7 @@ class DBUtilsExample(ExamplePage):
             '<tr><th>Name</th><th>Seminar</th><th>Paid</th>'
             '<th>%s</th></tr>' % button)
         for name, id, title, paid in result:
-            paid = paid and 'Yes' or 'No'
+            paid = 'Yes' if paid else 'No'
             id = id.ljust(4) + name
             wr('<tr><td>%s</td><td>%s</td><td>%s</td>'
                 '<td><input type="checkbox" name="id" value="%s"></td>'
@@ -326,7 +328,7 @@ class DBUtilsExample(ExamplePage):
         if not db:
             return
         cmd = ('insert into seminars values (%s,%s,%s,%s)'
-            % tuple(map(self.sqlEncode, values)))
+               % tuple(map(self.sqlEncode, values)))
         try:
             if self.dbapi_name == 'pg':
                 db.query('begin')
@@ -335,7 +337,7 @@ class DBUtilsExample(ExamplePage):
             else:
                 db.cursor().execute(cmd)
                 db.commit()
-        except self.dbapi.Error, error:
+        except self.dbapi.Error as error:
             if self.dbapi_name == 'pg':
                 db.query('end')
             else:
@@ -349,8 +351,8 @@ class DBUtilsExample(ExamplePage):
         db = self.connection()
         if not db:
             return
-        query = ('select id, title from seminars '
-            'where places_left is null or places_left>0 order by title')
+        query = ('select id, title from seminars'
+                 ' where places_left is null or places_left>0 order by title')
         try:
             if self.dbapi_name == 'pg':
                 result = db.query(query).getresult()
@@ -359,7 +361,7 @@ class DBUtilsExample(ExamplePage):
                 cursor.execute(query)
                 result = cursor.fetchall()
                 cursor.close()
-        except self.dbapi.Error, error:
+        except self.dbapi.Error as error:
             self.outputMsg(error, True)
             return
         if not result:
@@ -396,33 +398,33 @@ class DBUtilsExample(ExamplePage):
         try:
             if self.dbapi_name == 'pg':
                 db.query('begin')
-                cmd = ("update seminars set places_left=places_left-1 "
-                    "where id=%s" % self.sqlEncode(values[1]))
+                cmd = ('update seminars set places_left=places_left-1'
+                       ' where id=%s' % self.sqlEncode(values[1]))
                 db.query(cmd)
-                cmd = ("select places_left from seminars "
-                    "where id=%s" % self.sqlEncode(values[1]))
+                cmd = ('select places_left from seminars'
+                       ' where id=%s' % self.sqlEncode(values[1]))
                 if (db.query(cmd).getresult()[0][0] or 0) < 0:
                     raise self.dbapi.Error("No more places left.")
-                cmd = ("insert into attendees values (%s,%s,%s)"
-                    % tuple(map(self.sqlEncode, values)))
+                cmd = ('insert into attendees values (%s,%s,%s)'
+                       % tuple(map(self.sqlEncode, values)))
                 db.query(cmd)
                 db.query('end')
             else:
                 cursor = db.cursor()
-                cmd = ("update seminars set places_left=places_left-1 "
-                    "where id=%s" % self.sqlEncode(values[1]))
+                cmd = ('update seminars set places_left=places_left-1'
+                       ' where id=%s' % self.sqlEncode(values[1]))
                 cursor.execute(cmd)
-                cmd = ("select places_left from seminars "
-                    "where id=%s" % self.sqlEncode(values[1]))
+                cmd = ('select places_left from seminars'
+                       ' where id=%s' % self.sqlEncode(values[1]))
                 cursor.execute(cmd)
                 if (cursor.fetchone()[0] or 0) < 0:
                     raise self.dbapi.Error("No more places left.")
-                cmd = ("insert into attendees values (%s,%s,%s)"
-                    % tuple(map(self.sqlEncode, values)))
+                cmd = ('insert into attendees values (%s,%s,%s)'
+                       % tuple(map(self.sqlEncode, values)))
                 db.cursor().execute(cmd)
                 cursor.close()
                 db.commit()
-        except self.dbapi.Error, error:
+        except self.dbapi.Error as error:
             if self.dbapi_name == 'pg':
                 db.query('end')
             else:
