@@ -16,18 +16,18 @@ be replaced and errors passed on until the transaction is completed.
 
 Measures are taken to make the database connections thread-affine.
 This means the same thread always uses the same cached connection,
-and no other thread will use it. So the fact that the classic PyGreSQL
+and no other thread will use it.  So the fact that the classic PyGreSQL
 pg module is not thread-safe at the connection level is no problem here.
 
 For best performance, the application server should keep threads persistent.
 For this, you have to set MinServerThreads = MaxServerThreads in Webware.
 
 For more information on PostgreSQL, see:
-    http://www.postgresql.org
+    https://www.postgresql.org/
 For more information on PyGreSQL, see:
     http://www.pygresql.org
 For more information on Webware for Python, see:
-    http://www.webwareforpython.org
+    https://webwareforpython.github.io/w4py/
 
 
 Usage:
@@ -63,20 +63,20 @@ request database connections of that kind:
     db = persist.connection()
 
 You can use these connections just as if they were ordinary
-classic PyGreSQL API connections. Actually what you get is the
+classic PyGreSQL API connections.  Actually what you get is the
 hardened SteadyPg version of a classic PyGreSQL connection.
 
 Closing a persistent connection with db.close() will be silently
 ignored since it would be reopened at the next usage anyway and
-contrary to the intent of having persistent connections. Instead,
+contrary to the intent of having persistent connections.  Instead,
 the connection will be automatically closed when the thread dies.
 You can change this behavior be setting the closeable parameter.
 
 Note that you need to explicitly start transactions by calling the
-begin() method. This ensures that the transparent reopening will be
+begin() method.  This ensures that the transparent reopening will be
 suspended until the end of the transaction, and that the connection
-will be rolled back before being reused in the same thread. To end
-transactions, use on of the end(), commit() or rollback() methods.
+will be rolled back before being reused in the same thread.  To end
+transactions, use one of the end(), commit() or rollback() methods.
 
 By setting the threadlocal parameter to threading.local, getting
 connections may become a bit faster, but this may not work in all
@@ -84,11 +84,6 @@ environments (for instance, mod_wsgi is known to cause problems
 since it clears the threading.local data between requests).
 
 
-Requirements:
-
-Python >= 2.3, < 3.0, PyGreSQL >= 3.4.
-
-
 Ideas for improvement:
 
 * Add a thread for monitoring, restarting (or closing) bad or expired
@@ -103,18 +98,23 @@ Copyright, credits and license:
 * Based on an idea presented on the Webware developer mailing list
   by Geoffrey Talvola in July 2005
 
-Licensed under the Open Software License version 2.1.
+Licensed under the MIT license.
 
 """
 
-__version__ = '1.1'
-__revision__ = "$Rev: 8218 $"
-__date__ = "$Date: 2011-08-14 13:57:11 +0200 (So, 14. Aug 2011) $"
-
-
-import ThreadingLocal
 from DBUtils.SteadyPg import SteadyPgConnection
 
+__version__ = '1.4'
+
+try:
+    # Prefer the pure Python version of threading.local.
+    # The C implementation turned out to be problematic with mod_wsgi,
+    # since it does not keep the thread-local data between requests.
+    from _threading_local import local
+except ImportError:
+    # Fall back to the default version of threading.local.
+    from threading import local
+
 
 class PersistentPg:
     """Generator for persistent classic PyGreSQL connections.
@@ -126,7 +126,8 @@ class PersistentPg:
 
     version = __version__
 
-    def __init__(self, maxusage=None, setsession=None,
+    def __init__(
+            self, maxusage=None, setsession=None,
             closeable=False, threadlocal=None, *args, **kwargs):
         """Set up the persistent PostgreSQL connection generator.
 
@@ -149,7 +150,7 @@ class PersistentPg:
         self._setsession = setsession
         self._closeable = closeable
         self._args, self._kwargs = args, kwargs
-        self.thread = (threadlocal or ThreadingLocal.local)()
+        self.thread = (threadlocal or local)()
 
     def steady_connection(self):
         """Get a steady, non-persistent PyGreSQL connection."""
