@@ -8,7 +8,7 @@ closed or the database connection has been lost or when
 they are used more often than an optional usage limit.
 Database cursors are transparently reopened as well when
 the execution of a database operation cannot be performed
-due to a lost connection. Only if the connection is lost
+due to a lost connection.  Only if the connection is lost
 after the execution, when rows are already fetched from the
 database, this will give an error and the cursor will not
 be reopened automatically, because there is no reliable way
@@ -18,7 +18,7 @@ with a begin() call will not be silently replaced either.
 
 A typical situation where database connections are lost
 is when the database server or an intervening firewall is
-shutdown and restarted for maintenance reasons. In such a
+shutdown and restarted for maintenance reasons.  In such a
 case, all database connections would become unusable, even
 though the database service may be already available again.
 
@@ -33,9 +33,9 @@ Note, however, that the connections themselves may not be
 thread-safe (depending on the used DB-API module).
 
 For the Python DB-API 2 specification, see:
-    http://www.python.org/peps/pep-0249.html
+    https://www.python.org/dev/peps/pep-0249/
 For information on Webware for Python, see:
-    http://www.webwareforpython.org
+    https://webwareforpython.github.io/w4py/
 
 Usage:
 
@@ -44,7 +44,7 @@ way as you would use the connection constructor of a DB-API 2
 module if you specify the DB-API 2 module to be used as the
 first parameter, or alternatively you can specify an arbitrary
 constructor function returning new DB-API 2 compliant connection
-objects as the first parameter. Passing just a function allows
+objects as the first parameter.  Passing just a function allows
 implementing failover mechanisms and load balancing strategies.
 
 You may also specify a usage limit as the second parameter
@@ -57,7 +57,7 @@ When the connection to the database is lost or has been used
 too often, it will be transparently reset in most situations,
 without further notice.
 
-    import pgdb # import used DB-API 2 module
+    import pgdb  # import used DB-API 2 module
     from DBUtils.SteadyDB import connect
     db = connect(pgdb, 10000, ["set datestyle to german"],
         host=..., database=..., user=..., ...)
@@ -86,26 +86,30 @@ Copyright, credits and license:
 * Allowing creator functions as first parameter as in SQLAlchemy
   suggested by Ezio Vernacotola in December 2006
 
-Licensed under the Open Software License version 2.1.
+Licensed under the MIT license.
 
 """
 
-__version__ = '1.1'
-__revision__ = "$Rev: 8218 $"
-__date__ = "$Date: 2011-08-14 13:57:11 +0200 (So, 14. Aug 2011) $"
-
-
 import sys
 
+__version__ = '1.4'
+
+try:
+    baseint = (int, long)
+except NameError:  # Python 3
+    baseint = int
+
 
 class SteadyDBError(Exception):
     """General SteadyDB error."""
 
+
 class InvalidCursor(SteadyDBError):
     """Database cursor is invalid."""
 
 
-def connect(creator, maxusage=None, setsession=None,
+def connect(
+        creator, maxusage=None, setsession=None,
         failures=None, ping=1, closeable=True, *args, **kwargs):
     """A tough version of the connection constructor of a DB-API 2 module.
 
@@ -130,7 +134,8 @@ def connect(creator, maxusage=None, setsession=None,
         function or the connection constructor of the DB-API 2 module
 
     """
-    return SteadyDBConnection(creator, maxusage, setsession,
+    return SteadyDBConnection(
+        creator, maxusage, setsession,
         failures, ping, closeable, *args, **kwargs)
 
 
@@ -139,7 +144,8 @@ class SteadyDBConnection:
 
     version = __version__
 
-    def __init__(self, creator, maxusage=None, setsession=None,
+    def __init__(
+            self, creator, maxusage=None, setsession=None,
             failures=None, ping=1, closeable=True, *args, **kwargs):
         """Create a "tough" DB-API 2 connection."""
         # basic initialization to make finalizer work
@@ -172,7 +178,7 @@ class SteadyDBConnection:
             raise TypeError("%r is not a connection provider." % (creator,))
         if maxusage is None:
             maxusage = 0
-        if not isinstance(maxusage, (int, long)):
+        if not isinstance(maxusage, baseint):
             raise TypeError("'maxusage' must be an integer value.")
         self._maxusage = maxusage
         self._setsession_sql = setsession
@@ -180,11 +186,26 @@ class SteadyDBConnection:
                 failures, tuple) and not issubclass(failures, Exception):
             raise TypeError("'failures' must be a tuple of exceptions.")
         self._failures = failures
-        self._ping = isinstance(ping, int) and ping or 0
+        self._ping = ping if isinstance(ping, int) else 0
         self._closeable = closeable
         self._args, self._kwargs = args, kwargs
         self._store(self._create())
 
+    def __enter__(self):
+        """Enter the runtime context for the connection object."""
+        return self
+
+    def __exit__(self, *exc):
+        """Exit the runtime context for the connection object.
+
+        This does not close the connection, but it ends a transaction.
+
+        """
+        if exc[0] is None and exc[1] is None and exc[2] is None:
+            self.commit()
+        else:
+            self.rollback()
+
     def _create(self):
         """Create a new connection using the creator function."""
         con = self._creator(*self._args, **self._kwargs)
@@ -243,16 +264,18 @@ class SteadyDBConnection:
                         pass
             if self._failures is None:
                 try:
-                    self._failures = (self._dbapi.OperationalError,
+                    self._failures = (
+                        self._dbapi.OperationalError,
                         self._dbapi.InternalError)
                 except AttributeError:
                     try:
-                        self._failures = (self._creator.OperationalError,
+                        self._failures = (
+                            self._creator.OperationalError,
                             self._creator.InternalError)
                     except AttributeError:
                         try:
-                            self._failures = (con.OperationalError,
-                                con.InternalError)
+                            self._failures = (
+                                con.OperationalError, con.InternalError)
                         except AttributeError:
                             raise AttributeError(
                                 "Could not determine failure exceptions"
@@ -262,14 +285,14 @@ class SteadyDBConnection:
             else:
                 self._failure = self._failures
             self._setsession(con)
-        except Exception, error:
+        except Exception as error:
             # the database module could not be determined
             # or the session could not be prepared
-            try: # close the connection first
+            try:  # close the connection first
                 con.close()
             except Exception:
                 pass
-            raise error # reraise the original error again
+            raise error  # re-raise the original error again
         return con
 
     def _setsession(self, con=None):
@@ -310,7 +333,7 @@ class SteadyDBConnection:
         Rollback if forced or the connection was in a transaction.
 
         """
-        if force or self._transaction:
+        if not self._closed and (force or self._transaction):
             try:
                 self.rollback()
             except Exception:
@@ -325,10 +348,13 @@ class SteadyDBConnection:
 
         """
         if ping & self._ping:
-            try: # if possible, ping the connection
-                alive = self._con.ping()
+            try:  # if possible, ping the connection
+                try:  # pass a reconnect=False flag if this is supported
+                    alive = self._con.ping(False)
+                except TypeError:  # the reconnect flag is not supported
+                    alive = self._con.ping()
             except (AttributeError, IndexError, TypeError, ValueError):
-                self._ping = 0 # ping() is not available
+                self._ping = 0  # ping() is not available
                 alive = None
                 reconnect = False
             except Exception:
@@ -339,7 +365,7 @@ class SteadyDBConnection:
                 if alive:
                     reconnect = False
             if reconnect and not self._transaction:
-                try: # try to reopen the connection
+                try:  # try to reopen the connection
                     con = self._create()
                 except Exception:
                     pass
@@ -352,7 +378,8 @@ class SteadyDBConnection:
     def dbapi(self):
         """Return the underlying DB-API 2 module of the connection."""
         if self._dbapi is None:
-            raise AttributeError("Could not determine DB-API 2 module"
+            raise AttributeError(
+                "Could not determine DB-API 2 module"
                 " (please set creator.dbapi).")
         return self._dbapi
 
@@ -360,7 +387,8 @@ class SteadyDBConnection:
         """Return the thread safety level of the connection."""
         if self._threadsafety is None:
             if self._dbapi is None:
-                raise AttributeError("Could not determine threadsafety"
+                raise AttributeError(
+                    "Could not determine threadsafety"
                     " (please set creator.dbapi or creator.threadsafety).")
             return 0
         return self._threadsafety
@@ -372,7 +400,7 @@ class SteadyDBConnection:
         and it will not complain if you close it more than once.
 
         You can disallow closing connections by setting
-        the closeable parameter to something false. In this case,
+        the closeable parameter to something false.  In this case,
         closing tough connections will be silently ignored.
 
         """
@@ -384,7 +412,7 @@ class SteadyDBConnection:
     def begin(self, *args, **kwargs):
         """Indicate the beginning of a transaction.
 
-        During a transaction, connections will not not transparently
+        During a transaction, connections won't be transparently
         replaced, and all errors will be raised to the application.
 
         If the underlying driver supports this method, it will be called
@@ -402,12 +430,32 @@ class SteadyDBConnection:
     def commit(self):
         """Commit any pending transaction."""
         self._transaction = False
-        self._con.commit()
+        try:
+            self._con.commit()
+        except self._failures as error:  # cannot commit
+            try:  # try to reopen the connection
+                con = self._create()
+            except Exception:
+                pass
+            else:
+                self._close()
+                self._store(con)
+            raise error  # re-raise the original error
 
     def rollback(self):
         """Rollback pending transaction."""
         self._transaction = False
-        self._con.rollback()
+        try:
+            self._con.rollback()
+        except self._failures as error:  # cannot rollback
+            try:  # try to reopen the connection
+                con = self._create()
+            except Exception:
+                pass
+            else:
+                self._close()
+                self._store(con)
+            raise error  # re-raise the original error
 
     def cancel(self):
         """Cancel a long-running transaction.
@@ -435,18 +483,18 @@ class SteadyDBConnection:
         if not transaction:
             self._ping_check(2)
         try:
-            if self._maxusage:
-                if self._usage >= self._maxusage:
-                    # the connection was used too often
-                    raise self._failure
-            cursor = self._con.cursor(*args, **kwargs) # try to get a cursor
-        except self._failures, error: # error in getting cursor
-            try: # try to reopen the connection
+            # check whether the connection has been used too often
+            if (self._maxusage and self._usage >= self._maxusage
+                    and not transaction):
+                raise self._failure
+            cursor = self._con.cursor(*args, **kwargs)  # try to get a cursor
+        except self._failures as error:  # error in getting cursor
+            try:  # try to reopen the connection
                 con = self._create()
             except Exception:
                 pass
             else:
-                try: # and try one more time to get a cursor
+                try:  # and try one more time to get a cursor
                     cursor = con.cursor(*args, **kwargs)
                 except Exception:
                     pass
@@ -454,7 +502,7 @@ class SteadyDBConnection:
                     self._close()
                     self._store(con)
                     if transaction:
-                        raise error # reraise the original error again
+                        raise error  # re-raise the original error again
                     return cursor
                 try:
                     con.close()
@@ -462,7 +510,7 @@ class SteadyDBConnection:
                     pass
             if transaction:
                 self._transaction = False
-            raise error # reraise the original error again
+            raise error  # re-raise the original error again
         return cursor
 
     def cursor(self, *args, **kwargs):
@@ -472,7 +520,7 @@ class SteadyDBConnection:
     def __del__(self):
         """Delete the steady connection."""
         try:
-            self._close() # make sure the connection is closed
+            self._close()  # make sure the connection is closed
         except Exception:
             pass
 
@@ -495,6 +543,14 @@ class SteadyDBCursor:
             raise TypeError("%r is not a SteadyDBConnection." % (con,))
         self._closed = False
 
+    def __enter__(self):
+        """Enter the runtime context for the cursor object."""
+        return self
+
+    def __exit__(self, *exc):
+        """Exit the runtime context for the cursor object."""
+        self.close()
+
     def setinputsizes(self, sizes):
         """Store input sizes in case cursor needs to be reopened."""
         self._inputsizes = sizes
@@ -514,7 +570,7 @@ class SteadyDBCursor:
             cursor = self._cursor
         if self._inputsizes:
             cursor.setinputsizes(self._inputsizes)
-        for column, size in self._outputsizes.iteritems():
+        for column, size in self._outputsizes.items():
             if column is None:
                 cursor.setoutputsize(size)
             else:
@@ -542,25 +598,25 @@ class SteadyDBCursor:
             if not transaction:
                 con._ping_check(4)
             try:
-                if con._maxusage:
-                    if con._usage >= con._maxusage:
-                        # the connection was used too often
-                        raise con._failure
+                # check whether the connection has been used too often
+                if (con._maxusage and con._usage >= con._maxusage
+                        and not transaction):
+                    raise con._failure
                 if execute:
                     self._setsizes()
                 method = getattr(self._cursor, name)
-                result = method(*args, **kwargs) # try to execute
+                result = method(*args, **kwargs)  # try to execute
                 if execute:
                     self._clearsizes()
-            except con._failures, error: # execution error
+            except con._failures as error:  # execution error
                 if not transaction:
                     try:
                         cursor2 = con._cursor(
-                            *self._args, **self._kwargs) # open new cursor
+                            *self._args, **self._kwargs)  # open new cursor
                     except Exception:
                         pass
                     else:
-                        try: # and try one more time to execute
+                        try:  # and try one more time to execute
                             if execute:
                                 self._setsizes(cursor2)
                             method = getattr(cursor2, name)
@@ -578,14 +634,14 @@ class SteadyDBCursor:
                             cursor2.close()
                         except Exception:
                             pass
-                try: # try to reopen the connection
+                try:  # try to reopen the connection
                     con2 = con._create()
                 except Exception:
                     pass
                 else:
                     try:
                         cursor2 = con2.cursor(
-                            *self._args, **self._kwargs) # open new cursor
+                            *self._args, **self._kwargs)  # open new cursor
                     except Exception:
                         pass
                     else:
@@ -594,29 +650,31 @@ class SteadyDBCursor:
                             con._close()
                             con._store(con2)
                             self._cursor = cursor2
-                            raise error # raise the original error again
-                        try: # try one more time to execute
+                            raise error  # raise the original error again
+                        error2 = None
+                        try:  # try one more time to execute
                             if execute:
                                 self._setsizes(cursor2)
                             method2 = getattr(cursor2, name)
                             result = method2(*args, **kwargs)
                             if execute:
                                 self._clearsizes()
-                        except error.__class__: # same execution error
+                        except error.__class__:  # same execution error
                             use2 = False
-                        except Exception, error: # other execution errors
+                            error2 = error
+                        except Exception as error:  # other execution errors
                             use2 = True
+                            error2 = error
                         else:
                             use2 = True
-                            error = None
                         if use2:
                             self.close()
                             con._close()
                             con._store(con2)
                             self._cursor = cursor2
                             con._usage += 1
-                            if error:
-                                raise error # raise the other error
+                            if error2:
+                                raise error2  # raise the other error
                             return result
                         try:
                             cursor2.close()
@@ -628,7 +686,7 @@ class SteadyDBCursor:
                         pass
                 if transaction:
                     self._transaction = False
-                raise error # reraise the original error again
+                raise error  # re-raise the original error again
             else:
                 con._usage += 1
                 return result
@@ -637,7 +695,7 @@ class SteadyDBCursor:
     def __getattr__(self, name):
         """Inherit methods and attributes of underlying cursor."""
         if self._cursor:
-            if name.startswith('execute') or name.startswith('call'):
+            if name.startswith(('execute', 'call')):
                 # make execution methods "tough"
                 return self._get_tough_method(name)
             else:
@@ -648,6 +706,6 @@ class SteadyDBCursor:
     def __del__(self):
         """Delete the steady cursor."""
         try:
-            self.close() # make sure the cursor is closed
+            self.close()  # make sure the cursor is closed
         except Exception:
             pass
