@@ -11,18 +11,14 @@ Copyright and credit info:
 
 """
 
-__version__ = '1.1'
-__revision__ = "$Rev: 8218 $"
-__date__ = "$Date: 2011-08-14 13:57:11 +0200 (So, 14. Aug 2011) $"
-
-
-import sys
 import unittest
 
-sys.path.insert(1, '../..')
-# The TestSteadyDB module serves as a mock object for the DB-API 2 module:
-from DBUtils.Tests import TestSteadyDB as dbapi
-from DBUtils.PooledDB import PooledDB, TooManyConnections, SharedDBConnection
+import DBUtils.Tests.mock_db as dbapi
+
+from DBUtils.PooledDB import (
+    PooledDB, SharedDBConnection, InvalidConnection, TooManyConnections)
+
+__version__ = '1.4'
 
 
 class TestPooledDB(unittest.TestCase):
@@ -44,59 +40,59 @@ class TestPooledDB(unittest.TestCase):
         for threadsafety in (1, 2, 3):
             dbapi.threadsafety = threadsafety
             pool = PooledDB(dbapi, 0, 0, 1)
-            self.assert_(hasattr(pool, '_maxshared'))
+            self.assertTrue(hasattr(pool, '_maxshared'))
             if threadsafety > 1:
                 self.assertEqual(pool._maxshared, 1)
-                self.assert_(hasattr(pool, '_shared_cache'))
+                self.assertTrue(hasattr(pool, '_shared_cache'))
             else:
                 self.assertEqual(pool._maxshared, 0)
-                self.assert_(not hasattr(pool, '_shared_cache'))
+                self.assertFalse(hasattr(pool, '_shared_cache'))
 
     def test03_CreateConnection(self):
         for threadsafety in (1, 2):
             dbapi.threadsafety = threadsafety
             shareable = threadsafety > 1
-            pool = PooledDB(dbapi,
-                1, 1, 1, 0, False, None, None, True, None, None,
+            pool = PooledDB(
+                dbapi, 1, 1, 1, 0, False, None, None, True, None, None,
                 'PooledDBTestDB', user='PooledDBTestUser')
-            self.assert_(hasattr(pool, '_idle_cache'))
+            self.assertTrue(hasattr(pool, '_idle_cache'))
             self.assertEqual(len(pool._idle_cache), 1)
             if shareable:
-                self.assert_(hasattr(pool, '_shared_cache'))
+                self.assertTrue(hasattr(pool, '_shared_cache'))
                 self.assertEqual(len(pool._shared_cache), 0)
             else:
-                self.assert_(not hasattr(pool, '_shared_cache'))
-            self.assert_(hasattr(pool, '_maxusage'))
-            self.assertEqual(pool._maxusage, None)
-            self.assert_(hasattr(pool, '_setsession'))
-            self.assert_(pool._setsession is None)
+                self.assertFalse(hasattr(pool, '_shared_cache'))
+            self.assertTrue(hasattr(pool, '_maxusage'))
+            self.assertIsNone(pool._maxusage)
+            self.assertTrue(hasattr(pool, '_setsession'))
+            self.assertIsNone(pool._setsession)
             con = pool._idle_cache[0]
             from DBUtils.SteadyDB import SteadyDBConnection
-            self.assert_(isinstance(con, SteadyDBConnection))
-            self.assert_(hasattr(con, '_maxusage'))
+            self.assertTrue(isinstance(con, SteadyDBConnection))
+            self.assertTrue(hasattr(con, '_maxusage'))
             self.assertEqual(con._maxusage, 0)
-            self.assert_(hasattr(con, '_setsession_sql'))
-            self.assert_(con._setsession_sql is None)
+            self.assertTrue(hasattr(con, '_setsession_sql'))
+            self.assertIsNone(con._setsession_sql)
             db = pool.connection()
             self.assertEqual(len(pool._idle_cache), 0)
             if shareable:
                 self.assertEqual(len(pool._shared_cache), 1)
-            self.assert_(hasattr(db, '_con'))
+            self.assertTrue(hasattr(db, '_con'))
             self.assertEqual(db._con, con)
-            self.assert_(hasattr(db, 'cursor'))
-            self.assert_(hasattr(db, '_usage'))
+            self.assertTrue(hasattr(db, 'cursor'))
+            self.assertTrue(hasattr(db, '_usage'))
             self.assertEqual(db._usage, 0)
-            self.assert_(hasattr(con, '_con'))
+            self.assertTrue(hasattr(con, '_con'))
             db_con = con._con
-            self.assert_(hasattr(db_con, 'database'))
+            self.assertTrue(hasattr(db_con, 'database'))
             self.assertEqual(db_con.database, 'PooledDBTestDB')
-            self.assert_(hasattr(db_con, 'user'))
+            self.assertTrue(hasattr(db_con, 'user'))
             self.assertEqual(db_con.user, 'PooledDBTestUser')
-            self.assert_(hasattr(db_con, 'open_cursors'))
+            self.assertTrue(hasattr(db_con, 'open_cursors'))
             self.assertEqual(db_con.open_cursors, 0)
-            self.assert_(hasattr(db_con, 'num_uses'))
+            self.assertTrue(hasattr(db_con, 'num_uses'))
             self.assertEqual(db_con.num_uses, 0)
-            self.assert_(hasattr(db_con, 'num_queries'))
+            self.assertTrue(hasattr(db_con, 'num_queries'))
             self.assertEqual(db_con.num_queries, 0)
             cursor = db.cursor()
             self.assertEqual(db_con.open_cursors, 1)
@@ -118,8 +114,8 @@ class TestPooledDB(unittest.TestCase):
             self.assertEqual(db_con.open_cursors, 0)
             self.assertEqual(db_con.num_queries, 1)
             self.assertEqual(db._usage, 2)
-            self.assertEqual(db_con.session,
-                ['rollback', 'sessiontest'])
+            self.assertEqual(
+                db_con.session, ['rollback', 'sessiontest'])
             pool = PooledDB(dbapi, 1, 1, 1)
             self.assertEqual(len(pool._idle_cache), 1)
             if shareable:
@@ -146,8 +142,8 @@ class TestPooledDB(unittest.TestCase):
                 self.assertEqual(len(pool._shared_cache), 0)
             self.assertEqual(db._usage, 0)
             db_con = db._con._con
-            self.assert_(db_con.database is None)
-            self.assert_(db_con.user is None)
+            self.assertIsNone(db_con.database)
+            self.assertIsNone(db_con.user)
             db.close()
             self.assertEqual(len(pool._idle_cache), 1)
             if shareable:
@@ -158,8 +154,8 @@ class TestPooledDB(unittest.TestCase):
                 self.assertEqual(len(pool._shared_cache), 0)
             self.assertEqual(db._usage, 0)
             db_con = db._con._con
-            self.assert_(db_con.database is None)
-            self.assert_(db_con.user is None)
+            self.assertIsNone(db_con.database)
+            self.assertIsNone(db_con.user)
             db.close()
             self.assertEqual(len(pool._idle_cache), 1)
             if shareable:
@@ -175,41 +171,41 @@ class TestPooledDB(unittest.TestCase):
         for threadsafety in (1, 2):
             dbapi.threadsafety = threadsafety
             shareable = threadsafety > 1
-            pool = PooledDB(dbapi,
-                0, 1, 1, 0, False, None, None, True, None, None,
+            pool = PooledDB(
+                dbapi, 0, 1, 1, 0, False, None, None, True, None, None,
                 'PooledDBTestDB', user='PooledDBTestUser')
-            self.assert_(hasattr(pool, '_idle_cache'))
+            self.assertTrue(hasattr(pool, '_idle_cache'))
             self.assertEqual(len(pool._idle_cache), 0)
             db = pool.connection()
-            self.assert_(hasattr(db, '_con'))
+            self.assertTrue(hasattr(db, '_con'))
             con = db._con
             self.assertEqual(len(pool._idle_cache), 0)
             if shareable:
                 self.assertEqual(len(pool._shared_cache), 1)
-                self.assert_(hasattr(db, '_shared_con'))
+                self.assertTrue(hasattr(db, '_shared_con'))
                 shared_con = db._shared_con
                 self.assertEqual(pool._shared_cache[0], shared_con)
-                self.assert_(hasattr(shared_con, 'shared'))
+                self.assertTrue(hasattr(shared_con, 'shared'))
                 self.assertEqual(shared_con.shared, 1)
-                self.assert_(hasattr(shared_con, 'con'))
+                self.assertTrue(hasattr(shared_con, 'con'))
                 self.assertEqual(shared_con.con, con)
             from DBUtils.SteadyDB import SteadyDBConnection
-            self.assert_(isinstance(con, SteadyDBConnection))
-            self.assert_(hasattr(con, '_con'))
+            self.assertTrue(isinstance(con, SteadyDBConnection))
+            self.assertTrue(hasattr(con, '_con'))
             db_con = con._con
-            self.assert_(hasattr(db_con, 'num_queries'))
+            self.assertTrue(hasattr(db_con, 'num_queries'))
             self.assertEqual(db._usage, 0)
             self.assertEqual(db_con.num_queries, 0)
             db.cursor().execute('select test')
             self.assertEqual(db._usage, 1)
             self.assertEqual(db_con.num_queries, 1)
             db.close()
-            self.assertEqual(db._con, None)
+            self.assertIsNone(db._con)
             if shareable:
-                self.assertEqual(db._shared_con, None)
+                self.assertIsNone(db._shared_con)
                 self.assertEqual(shared_con.shared, 0)
-            self.assert_(not hasattr(db, '_usage'))
-            self.assert_(not hasattr(db_con, '_num_queries'))
+            self.assertRaises(InvalidConnection, getattr, db, '_usage')
+            self.assertFalse(hasattr(db_con, '_num_queries'))
             self.assertEqual(len(pool._idle_cache), 1)
             self.assertEqual(pool._idle_cache[0]._con, db_con)
             if shareable:
@@ -228,9 +224,9 @@ class TestPooledDB(unittest.TestCase):
                 self.assertEqual(shared_con.shared, 1)
             self.assertEqual(db._usage, 1)
             self.assertEqual(db_con.num_queries, 1)
-            self.assert_(hasattr(db_con, 'database'))
+            self.assertTrue(hasattr(db_con, 'database'))
             self.assertEqual(db_con.database, 'PooledDBTestDB')
-            self.assert_(hasattr(db_con, 'user'))
+            self.assertTrue(hasattr(db_con, 'user'))
             self.assertEqual(db_con.user, 'PooledDBTestUser')
             db.cursor().execute('select test')
             self.assertEqual(db_con.num_queries, 2)
@@ -261,8 +257,10 @@ class TestPooledDB(unittest.TestCase):
             self.assertEqual(len(pool._idle_cache), 0)
             pool = PooledDB(dbapi, 10)
             closed = ['no']
+
             def close(what=closed):
                 what[0] = 'yes'
+
             pool._idle_cache[7]._con.close = close
             self.assertEqual(closed, ['no'])
             del pool
@@ -285,10 +283,13 @@ class TestPooledDB(unittest.TestCase):
                 self.assertEqual(len(pool._shared_cache), 0)
             pool = PooledDB(dbapi, 10, 10, 5)
             closed = []
+
             def close_idle(what=closed):
                 what.append('idle')
+
             def close_shared(what=closed):
                 what.append('shared')
+
             if shareable:
                 cache = []
                 for i in range(5):
@@ -481,28 +482,34 @@ class TestPooledDB(unittest.TestCase):
             pool = PooledDB(dbapi)
             self.assertEqual(len(pool._idle_cache), 0)
             cache = [pool.connection() for i in range(10)]
+            self.assertEqual(len(cache), 10)
             self.assertEqual(len(pool._idle_cache), 0)
             pool = PooledDB(dbapi, 1, 1, 0)
             self.assertEqual(len(pool._idle_cache), 1)
             cache = [pool.connection() for i in range(10)]
+            self.assertEqual(len(cache), 10)
             self.assertEqual(len(pool._idle_cache), 0)
             pool = PooledDB(dbapi, 0, 0, 1)
             cache = [pool.connection() for i in range(10)]
+            self.assertEqual(len(cache), 10)
             self.assertEqual(len(pool._idle_cache), 0)
             if shareable:
                 self.assertEqual(len(pool._shared_cache), 1)
             pool = PooledDB(dbapi, 1, 1, 1)
             self.assertEqual(len(pool._idle_cache), 1)
             cache = [pool.connection() for i in range(10)]
+            self.assertEqual(len(cache), 10)
             self.assertEqual(len(pool._idle_cache), 0)
             if shareable:
                 self.assertEqual(len(pool._shared_cache), 1)
             pool = PooledDB(dbapi, 0, 0, 7)
             cache = [pool.connection(False) for i in range(3)]
+            self.assertEqual(len(cache), 3)
             self.assertEqual(len(pool._idle_cache), 0)
             if shareable:
                 self.assertEqual(len(pool._shared_cache), 0)
             cache = [pool.connection() for i in range(10)]
+            self.assertEqual(len(cache), 10)
             self.assertEqual(len(pool._idle_cache), 3)
             if shareable:
                 self.assertEqual(len(pool._shared_cache), 7)
@@ -516,15 +523,15 @@ class TestPooledDB(unittest.TestCase):
             db.cursor().execute('select test')
             cache.append(db)
         for i, db in enumerate(cache):
-            self.assertEqual(db._shared_con.shared, 2 <= i < 4 and 1 or 2)
+            self.assertEqual(db._shared_con.shared, 1 if 2 <= i < 4 else 2)
         cache[2].begin()
         cache[3].begin()
         db = pool.connection()
-        self.assert_(db._con is cache[0]._con)
+        self.assertIs(db._con, cache[0]._con)
         db.close()
         cache[3].rollback()
         db = pool.connection()
-        self.assert_(db._con is cache[3]._con)
+        self.assertIs(db._con, cache[3]._con)
 
     def test10_EquallyShared(self):
         for threadsafety in (1, 2):
@@ -649,9 +656,9 @@ class TestPooledDB(unittest.TestCase):
             dbapi.threadsafety = threadsafety
             shareable = threadsafety > 1
             pool = PooledDB(dbapi, 1, 2, 2, 3)
-            self.assert_(hasattr(pool, '_maxconnections'))
+            self.assertTrue(hasattr(pool, '_maxconnections'))
             self.assertEqual(pool._maxconnections, 3)
-            self.assert_(hasattr(pool, '_connections'))
+            self.assertTrue(hasattr(pool, '_connections'))
             self.assertEqual(pool._connections, 0)
             self.assertEqual(len(pool._idle_cache), 1)
             cache = []
@@ -816,17 +823,19 @@ class TestPooledDB(unittest.TestCase):
             db = pool.connection(False)
             self.assertEqual(pool._connections, 1)
             self.assertEqual(len(pool._idle_cache), 0)
+
             def connection():
                 db = pool.connection()
                 cursor = db.cursor()
                 cursor.execute('set thread')
                 cursor.close()
                 db.close()
+
             from threading import Thread
             thread = Thread(target=connection)
             thread.start()
             thread.join(0.1)
-            self.assert_(thread.isAlive())
+            self.assertTrue(thread.is_alive())
             self.assertEqual(pool._connections, 1)
             self.assertEqual(len(pool._idle_cache), 0)
             if shareable:
@@ -835,7 +844,7 @@ class TestPooledDB(unittest.TestCase):
             self.assertEqual(session, ['rollback'])
             del db
             thread.join(0.1)
-            self.assert_(not thread.isAlive())
+            self.assertFalse(thread.is_alive())
             self.assertEqual(pool._connections, 0)
             self.assertEqual(len(pool._idle_cache), 1)
             if shareable:
@@ -843,8 +852,9 @@ class TestPooledDB(unittest.TestCase):
             db = pool.connection(False)
             self.assertEqual(pool._connections, 1)
             self.assertEqual(len(pool._idle_cache), 0)
-            self.assertEqual(session, ['rollback',
-                'rollback', 'thread', 'rollback'])
+            self.assertEqual(
+                session, ['rollback', 'rollback', 'thread', 'rollback'])
+            del db
 
     def test14_MaxUsage(self):
         for threadsafety in (1, 2):
@@ -861,7 +871,7 @@ class TestPooledDB(unittest.TestCase):
                 self.assertEqual(db._con._con.num_uses, 0)
                 self.assertEqual(db._con._con.num_queries, 0)
                 for i in range(20):
-                    cursor=db.cursor()
+                    cursor = db.cursor()
                     self.assertEqual(db._con._con.open_cursors, 1)
                     cursor.execute('select test%i' % i)
                     r = cursor.fetchone()
@@ -889,25 +899,26 @@ class TestPooledDB(unittest.TestCase):
             self.assertEqual(pool._setsession, setsession)
             db = pool.connection(False)
             self.assertEqual(db._setsession_sql, setsession)
-            self.assertEqual(db._con._con.session,
-                ['time zone', 'datestyle'])
+            self.assertEqual(
+                db._con._con.session, ['time zone', 'datestyle'])
             db.cursor().execute('select test')
             db.cursor().execute('set test1')
             self.assertEqual(db._usage, 2)
             self.assertEqual(db._con._con.num_uses, 4)
             self.assertEqual(db._con._con.num_queries, 1)
-            self.assertEqual(db._con._con.session,
-                ['time zone', 'datestyle', 'test1'])
+            self.assertEqual(
+                db._con._con.session, ['time zone', 'datestyle', 'test1'])
             db.close()
             db = pool.connection(False)
             self.assertEqual(db._setsession_sql, setsession)
-            self.assertEqual(db._con._con.session,
+            self.assertEqual(
+                db._con._con.session,
                 ['time zone', 'datestyle', 'test1', 'rollback'])
             db._con._con.close()
             db.cursor().execute('select test')
             db.cursor().execute('set test2')
-            self.assertEqual(db._con._con.session,
-                ['time zone', 'datestyle', 'test2'])
+            self.assertEqual(
+                db._con._con.session, ['time zone', 'datestyle', 'test2'])
 
     def test16_OneThreadTwoConnections(self):
         for threadsafety in (1, 2):
@@ -970,13 +981,18 @@ class TestPooledDB(unittest.TestCase):
         for threadsafety in (1, 2):
             dbapi.threadsafety = threadsafety
             pool = PooledDB(dbapi, 2, 2, 0, 2, True)
-            from Queue import Queue, Empty
+            try:
+                from queue import Queue, Empty
+            except ImportError:  # Python 2
+                from Queue import Queue, Empty
             queue = Queue(3)
+
             def connection():
                 try:
                     queue.put(pool.connection(), 1, 1)
                 except Exception:
                     queue.put(pool.connection(), 1)
+
             from threading import Thread
             for i in range(3):
                 Thread(target=connection).start()
@@ -1030,63 +1046,63 @@ class TestPooledDB(unittest.TestCase):
         dbapi.threadsafety = 2
         pool = PooledDB(dbapi, 1, 1, 0, 0, False, None, None, True, None, 0)
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 0)
         db._con.close()
         db.close()
         db = pool.connection()
-        self.assert_(not db._con._con.valid)
+        self.assertFalse(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 0)
         pool = PooledDB(dbapi, 1, 1, 1, 0, False, None, None, True, None, 0)
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 0)
         db._con.close()
         db = pool.connection()
-        self.assert_(not db._con._con.valid)
+        self.assertFalse(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 0)
         pool = PooledDB(dbapi, 1, 1, 0, 0, False, None, None, True, None, 1)
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 1)
         db._con.close()
         db.close()
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 2)
         pool = PooledDB(dbapi, 1, 1, 1, 0, False, None, None, True, None, 1)
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 3)
         db._con.close()
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 4)
         pool = PooledDB(dbapi, 1, 1, 1, 0, False, None, None, True, None, 2)
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 4)
         db._con.close()
         db = pool.connection()
-        self.assert_(not db._con._con.valid)
+        self.assertFalse(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 4)
         db.cursor()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 5)
         pool = PooledDB(dbapi, 1, 1, 1, 0, False, None, None, True, None, 4)
         db = pool.connection()
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 5)
         db._con.close()
         db = pool.connection()
-        self.assert_(not db._con._con.valid)
+        self.assertFalse(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 5)
         cursor = db.cursor()
         db._con.close()
-        self.assert_(not db._con._con.valid)
+        self.assertFalse(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 5)
         cursor.execute('select test')
-        self.assert_(db._con._con.valid)
+        self.assertTrue(db._con._con.valid)
         self.assertEqual(Connection.num_pings, 6)
         Connection.has_ping = False
         Connection.num_pings = 0
@@ -1130,44 +1146,44 @@ class TestPooledDB(unittest.TestCase):
         pool = PooledDB(dbapi, 0, 2, 2)
         db1 = pool.connection()
         db2 = pool.connection()
-        self.assert_(db2._con is not db1._con)
+        self.assertIsNot(db2._con, db1._con)
         db2.close()
         db2 = pool.connection()
-        self.assert_(db2._con is not db1._con)
+        self.assertIsNot(db2._con, db1._con)
         db = pool.connection()
-        self.assert_(db._con is db1._con)
+        self.assertIs(db._con, db1._con)
         db.close()
         db1.begin()
         db = pool.connection()
-        self.assert_(db._con is db2._con)
+        self.assertIs(db._con, db2._con)
         db.close()
         db2.begin()
         pool.connection(False)
         self.assertRaises(TooManyConnections, pool.connection)
         db1.rollback()
         db = pool.connection()
-        self.assert_(db._con is db1._con)
+        self.assertIs(db._con, db1._con)
 
     def test21_ResetTransaction(self):
         pool = PooledDB(dbapi, 1, 1, 0)
         db = pool.connection()
         db.begin()
         con = db._con
-        self.assert_(con._transaction)
+        self.assertTrue(con._transaction)
         self.assertEqual(con._con.session, ['rollback'])
         db.close()
-        self.assert_(pool.connection()._con is con)
-        self.assert_(not con._transaction)
+        self.assertIs(pool.connection()._con, con)
+        self.assertFalse(con._transaction)
         self.assertEqual(con._con.session, ['rollback'] * 3)
         pool = PooledDB(dbapi, 1, 1, 0, reset=False)
         db = pool.connection()
         db.begin()
         con = db._con
-        self.assert_(con._transaction)
+        self.assertTrue(con._transaction)
         self.assertEqual(con._con.session, [])
         db.close()
-        self.assert_(pool.connection()._con is con)
-        self.assert_(not con._transaction)
+        self.assertIs(pool.connection()._con, con)
+        self.assertFalse(con._transaction)
         self.assertEqual(con._con.session, ['rollback'])
 
 
@@ -1196,26 +1212,26 @@ class TestSharedDBConnection(unittest.TestCase):
         con1.con._transaction = False
         con2 = SharedDBConnection(dbapi.connect())
         con2.con._transaction = False
-        self.assert_(con1 == con2)
-        self.assert_(con1 <= con2)
-        self.assert_(con1 >= con2)
-        self.assert_(not con1 != con2)
-        self.assert_(not con1 < con2)
-        self.assert_(not con1 > con2)
+        self.assertTrue(con1 == con2)
+        self.assertTrue(con1 <= con2)
+        self.assertTrue(con1 >= con2)
+        self.assertFalse(con1 != con2)
+        self.assertFalse(con1 < con2)
+        self.assertFalse(con1 > con2)
         con2.share()
-        self.assert_(not con1 == con2)
-        self.assert_(con1 <= con2)
-        self.assert_(not con1 >= con2)
-        self.assert_(con1 != con2)
-        self.assert_(con1 < con2)
-        self.assert_(not con1 > con2)
+        self.assertFalse(con1 == con2)
+        self.assertTrue(con1 <= con2)
+        self.assertFalse(con1 >= con2)
+        self.assertTrue(con1 != con2)
+        self.assertTrue(con1 < con2)
+        self.assertFalse(con1 > con2)
         con1.con._transaction = True
-        self.assert_(not con1 == con2)
-        self.assert_(not con1 <= con2)
-        self.assert_(con1 >= con2)
-        self.assert_(con1 != con2)
-        self.assert_(not con1 < con2)
-        self.assert_(con1 > con2)
+        self.assertFalse(con1 == con2)
+        self.assertFalse(con1 <= con2)
+        self.assertTrue(con1 >= con2)
+        self.assertTrue(con1 != con2)
+        self.assertFalse(con1 < con2)
+        self.assertTrue(con1 > con2)
 
 
 if __name__ == '__main__':
