@@ -10,145 +10,14 @@ Copyright and credit info:
 
 """
 
-__version__ = '1.1'
-__revision__ = "$Rev: 8218 $"
-__date__ = "$Date: 2011-08-14 13:57:11 +0200 (So, 14. Aug 2011) $"
-
-
-import sys
-
-# This module also serves as a mock object for the DB-API 2 module:
-
-dbapi = sys.modules[__name__]
-
-threadsafety = 2
-
-class Error(StandardError): pass
-class DatabaseError(Error): pass
-class OperationalError(DatabaseError): pass
-class InternalError(DatabaseError): pass
-class ProgrammingError(DatabaseError): pass
-
-
-def connect(database=None, user=None):
-    return Connection(database, user)
-
-
-class Connection:
-
-    has_ping = False
-    num_pings = 0
-
-    def __init__(self, database=None, user=None):
-        self.database = database
-        self.user = user
-        self.valid = False
-        if database == 'error':
-            raise OperationalError
-        self.open_cursors = 0
-        self.num_uses = 0
-        self.num_queries = 0
-        self.num_pings = 0
-        self.session = []
-        self.valid = True
-
-    def close(self):
-        if not self.valid:
-            raise InternalError
-        self.open_cursors = 0
-        self.num_uses = 0
-        self.num_queries = 0
-        self.session = []
-        self.valid = False
-
-    def commit(self):
-        self.session.append('commit')
-
-    def rollback(self):
-        self.session.append('rollback')
-
-    def ping(self):
-        cls = self.__class__
-        cls.num_pings += 1
-        if not cls.has_ping:
-            raise AttributeError
-        if not self.valid:
-            raise OperationalError
-
-    def cursor(self, name=None):
-        if not self.valid:
-            raise InternalError
-        return Cursor(self, name)
-
-
-class Cursor:
-
-    def __init__(self, con, name=None):
-        self.con = con
-        self.valid = False
-        if name == 'error':
-            raise OperationalError
-        self.result = None
-        self.inputsizes = []
-        self.outputsizes = {}
-        con.open_cursors += 1
-        self.valid = True
-
-    def close(self):
-        if not self.valid:
-            raise InternalError
-        self.con.open_cursors -= 1
-        self.valid = False
-
-    def execute(self, operation):
-        if not self.valid or not self.con.valid:
-            raise InternalError
-        self.con.num_uses += 1
-        if operation.startswith('select '):
-            self.con.num_queries += 1
-            self.result = operation[7:]
-        elif operation.startswith('set '):
-            self.con.session.append(operation[4:])
-            self.result = None
-        elif operation == 'get sizes':
-            self.result = (self.inputsizes, self.outputsizes)
-            self.inputsizes = []
-            self.outputsizes = {}
-        else:
-            raise ProgrammingError
-
-    def fetchone(self):
-        if not self.valid:
-            raise InternalError
-        result = self.result
-        self.result = None
-        return result
-
-    def callproc(self, procname):
-        if not self.valid or not self.con.valid:
-            raise InternalError
-        self.con.num_uses += 1
-
-    def setinputsizes(self, sizes):
-        if not self.valid:
-            raise InternalError
-        self.inputsizes = sizes
-
-    def setoutputsize(self, size, column=None):
-        if not self.valid:
-            raise InternalError
-        self.outputsizes[column] = size
-
-    def __del__(self):
-        if self.valid:
-            self.close()
-
-
 import unittest
 
-sys.path.insert(1, '../..')
-from DBUtils.SteadyDB import connect as SteadyDBconnect
-from DBUtils.SteadyDB import SteadyDBConnection, SteadyDBCursor
+import DBUtils.Tests.mock_db as dbapi
+
+from DBUtils.SteadyDB import (
+    connect as SteadyDBconnect, SteadyDBConnection, SteadyDBCursor)
+
+__version__ = '1.4'
 
 
 class TestSteadyDB(unittest.TestCase):
@@ -161,22 +30,22 @@ class TestSteadyDB(unittest.TestCase):
         self.assertEqual(SteadyDBConnection.version, __version__)
 
     def test01_MockedConnection(self):
-        db = connect('SteadyDBTestDB',
-            user='SteadyDBTestUser')
+        db = dbapi.connect(
+            'SteadyDBTestDB', user='SteadyDBTestUser')
         db.__class__.has_ping = False
         db.__class__.num_pings = 0
-        self.assert_(hasattr(db, 'database'))
+        self.assertTrue(hasattr(db, 'database'))
         self.assertEqual(db.database, 'SteadyDBTestDB')
-        self.assert_(hasattr(db, 'user'))
+        self.assertTrue(hasattr(db, 'user'))
         self.assertEqual(db.user, 'SteadyDBTestUser')
-        self.assert_(hasattr(db, 'cursor'))
-        self.assert_(hasattr(db, 'close'))
-        self.assert_(hasattr(db, 'open_cursors'))
-        self.assert_(hasattr(db, 'num_uses'))
-        self.assert_(hasattr(db, 'num_queries'))
-        self.assert_(hasattr(db, 'session'))
-        self.assert_(hasattr(db, 'valid'))
-        self.assert_(db.valid)
+        self.assertTrue(hasattr(db, 'cursor'))
+        self.assertTrue(hasattr(db, 'close'))
+        self.assertTrue(hasattr(db, 'open_cursors'))
+        self.assertTrue(hasattr(db, 'num_uses'))
+        self.assertTrue(hasattr(db, 'num_queries'))
+        self.assertTrue(hasattr(db, 'session'))
+        self.assertTrue(hasattr(db, 'valid'))
+        self.assertTrue(db.valid)
         self.assertEqual(db.open_cursors, 0)
         for i in range(3):
             cursor = db.cursor()
@@ -190,44 +59,44 @@ class TestSteadyDB(unittest.TestCase):
         del cursor
         self.assertEqual(db.open_cursors, 0)
         cursor = db.cursor()
-        self.assert_(hasattr(cursor, 'execute'))
-        self.assert_(hasattr(cursor, 'fetchone'))
-        self.assert_(hasattr(cursor, 'callproc'))
-        self.assert_(hasattr(cursor, 'close'))
-        self.assert_(hasattr(cursor, 'valid'))
-        self.assert_(cursor.valid)
+        self.assertTrue(hasattr(cursor, 'execute'))
+        self.assertTrue(hasattr(cursor, 'fetchone'))
+        self.assertTrue(hasattr(cursor, 'callproc'))
+        self.assertTrue(hasattr(cursor, 'close'))
+        self.assertTrue(hasattr(cursor, 'valid'))
+        self.assertTrue(cursor.valid)
         self.assertEqual(db.open_cursors, 1)
         for i in range(3):
             self.assertEqual(db.num_uses, i)
             self.assertEqual(db.num_queries, i)
             cursor.execute('select test%d' % i)
             self.assertEqual(cursor.fetchone(), 'test%d' % i)
-        self.assert_(cursor.valid)
+        self.assertTrue(cursor.valid)
         self.assertEqual(db.open_cursors, 1)
         for i in range(4):
             cursor.callproc('test')
         cursor.close()
-        self.assert_(not cursor.valid)
+        self.assertFalse(cursor.valid)
         self.assertEqual(db.open_cursors, 0)
         self.assertEqual(db.num_uses, 7)
         self.assertEqual(db.num_queries, 3)
-        self.assertRaises(InternalError, cursor.close)
-        self.assertRaises(InternalError, cursor.execute, 'select test')
-        self.assert_(db.valid)
-        self.assert_(not db.__class__.has_ping)
+        self.assertRaises(dbapi.InternalError, cursor.close)
+        self.assertRaises(dbapi.InternalError, cursor.execute, 'select test')
+        self.assertTrue(db.valid)
+        self.assertFalse(db.__class__.has_ping)
         self.assertEqual(db.__class__.num_pings, 0)
         self.assertRaises(AttributeError, db.ping)
         self.assertEqual(db.__class__.num_pings, 1)
         db.__class__.has_ping = True
-        self.assert_(db.ping() is None)
+        self.assertIsNone(db.ping())
         self.assertEqual(db.__class__.num_pings, 2)
         db.close()
-        self.assert_(not db.valid)
+        self.assertFalse(db.valid)
         self.assertEqual(db.num_uses, 0)
         self.assertEqual(db.num_queries, 0)
-        self.assertRaises(InternalError, db.close)
-        self.assertRaises(InternalError, db.cursor)
-        self.assertRaises(OperationalError, db.ping)
+        self.assertRaises(dbapi.InternalError, db.close)
+        self.assertRaises(dbapi.InternalError, db.cursor)
+        self.assertRaises(dbapi.OperationalError, db.ping)
         self.assertEqual(db.__class__.num_pings, 3)
         db.__class__.has_ping = False
         db.__class__.num_pings = 0
@@ -239,8 +108,8 @@ class TestSteadyDB(unittest.TestCase):
         for i in range(3):
             db.close()
         del db
-        self.assertRaises(OperationalError, SteadyDBconnect,
-            dbapi, database='error')
+        self.assertRaises(
+            dbapi.OperationalError, SteadyDBconnect, dbapi, database='error')
         db = SteadyDBconnect(dbapi, database='ok')
         cursor = db.cursor()
         for i in range(3):
@@ -248,42 +117,43 @@ class TestSteadyDB(unittest.TestCase):
         cursor = db.cursor('ok')
         for i in range(3):
             cursor.close()
-        self.assertRaises(OperationalError, db.cursor, 'error')
+        self.assertRaises(dbapi.OperationalError, db.cursor, 'error')
 
     def test03_Close(self):
         for closeable in (False, True):
             db = SteadyDBconnect(dbapi, closeable=closeable)
-            self.assert_(db._con.valid)
+            self.assertTrue(db._con.valid)
             db.close()
-            self.assert_(closeable ^ db._con.valid)
+            self.assertTrue(closeable ^ db._con.valid)
             db.close()
-            self.assert_(closeable ^ db._con.valid)
+            self.assertTrue(closeable ^ db._con.valid)
             db._close()
-            self.assert_(not db._con.valid)
+            self.assertFalse(db._con.valid)
             db._close()
-            self.assert_(not db._con.valid)
+            self.assertFalse(db._con.valid)
 
     def test04_Connection(self):
-        db = SteadyDBconnect(dbapi, 0, None, None, None, True,
+        db = SteadyDBconnect(
+            dbapi, 0, None, None, None, True,
             'SteadyDBTestDB', user='SteadyDBTestUser')
-        self.assert_(isinstance(db, SteadyDBConnection))
-        self.assert_(hasattr(db, '_con'))
-        self.assert_(hasattr(db, '_usage'))
+        self.assertTrue(isinstance(db, SteadyDBConnection))
+        self.assertTrue(hasattr(db, '_con'))
+        self.assertTrue(hasattr(db, '_usage'))
         self.assertEqual(db._usage, 0)
-        self.assert_(hasattr(db._con, 'valid'))
-        self.assert_(db._con.valid)
-        self.assert_(hasattr(db._con, 'cursor'))
-        self.assert_(hasattr(db._con, 'close'))
-        self.assert_(hasattr(db._con, 'open_cursors'))
-        self.assert_(hasattr(db._con, 'num_uses'))
-        self.assert_(hasattr(db._con, 'num_queries'))
-        self.assert_(hasattr(db._con, 'session'))
-        self.assert_(hasattr(db._con, 'database'))
+        self.assertTrue(hasattr(db._con, 'valid'))
+        self.assertTrue(db._con.valid)
+        self.assertTrue(hasattr(db._con, 'cursor'))
+        self.assertTrue(hasattr(db._con, 'close'))
+        self.assertTrue(hasattr(db._con, 'open_cursors'))
+        self.assertTrue(hasattr(db._con, 'num_uses'))
+        self.assertTrue(hasattr(db._con, 'num_queries'))
+        self.assertTrue(hasattr(db._con, 'session'))
+        self.assertTrue(hasattr(db._con, 'database'))
         self.assertEqual(db._con.database, 'SteadyDBTestDB')
-        self.assert_(hasattr(db._con, 'user'))
+        self.assertTrue(hasattr(db._con, 'user'))
         self.assertEqual(db._con.user, 'SteadyDBTestUser')
-        self.assert_(hasattr(db, 'cursor'))
-        self.assert_(hasattr(db, 'close'))
+        self.assertTrue(hasattr(db, 'cursor'))
+        self.assertTrue(hasattr(db, 'close'))
         self.assertEqual(db._con.open_cursors, 0)
         for i in range(3):
             cursor = db.cursor()
@@ -297,12 +167,12 @@ class TestSteadyDB(unittest.TestCase):
         del cursor
         self.assertEqual(db._con.open_cursors, 0)
         cursor = db.cursor()
-        self.assert_(hasattr(cursor, 'execute'))
-        self.assert_(hasattr(cursor, 'fetchone'))
-        self.assert_(hasattr(cursor, 'callproc'))
-        self.assert_(hasattr(cursor, 'close'))
-        self.assert_(hasattr(cursor, 'valid'))
-        self.assert_(cursor.valid)
+        self.assertTrue(hasattr(cursor, 'execute'))
+        self.assertTrue(hasattr(cursor, 'fetchone'))
+        self.assertTrue(hasattr(cursor, 'callproc'))
+        self.assertTrue(hasattr(cursor, 'close'))
+        self.assertTrue(hasattr(cursor, 'valid'))
+        self.assertTrue(cursor.valid)
         self.assertEqual(db._con.open_cursors, 1)
         for i in range(3):
             self.assertEqual(db._usage, i)
@@ -310,36 +180,36 @@ class TestSteadyDB(unittest.TestCase):
             self.assertEqual(db._con.num_queries, i)
             cursor.execute('select test%d' % i)
             self.assertEqual(cursor.fetchone(), 'test%d' % i)
-        self.assert_(cursor.valid)
+        self.assertTrue(cursor.valid)
         self.assertEqual(db._con.open_cursors, 1)
         for i in range(4):
             cursor.callproc('test')
         cursor.close()
-        self.assert_(not cursor.valid)
+        self.assertFalse(cursor.valid)
         self.assertEqual(db._con.open_cursors, 0)
         self.assertEqual(db._usage, 7)
         self.assertEqual(db._con.num_uses, 7)
         self.assertEqual(db._con.num_queries, 3)
         cursor.close()
         cursor.execute('select test8')
-        self.assert_(cursor.valid)
+        self.assertTrue(cursor.valid)
         self.assertEqual(db._con.open_cursors, 1)
         self.assertEqual(cursor.fetchone(), 'test8')
         self.assertEqual(db._usage, 8)
         self.assertEqual(db._con.num_uses, 8)
         self.assertEqual(db._con.num_queries, 4)
-        self.assert_(db._con.valid)
+        self.assertTrue(db._con.valid)
         db.close()
-        self.assert_(not db._con.valid)
+        self.assertFalse(db._con.valid)
         self.assertEqual(db._con.open_cursors, 0)
         self.assertEqual(db._usage, 8)
         self.assertEqual(db._con.num_uses, 0)
         self.assertEqual(db._con.num_queries, 0)
-        self.assertRaises(InternalError, db._con.close)
+        self.assertRaises(dbapi.InternalError, db._con.close)
         db.close()
-        self.assertRaises(InternalError, db._con.cursor)
+        self.assertRaises(dbapi.InternalError, db._con.cursor)
         cursor = db.cursor()
-        self.assert_(db._con.valid)
+        self.assertTrue(db._con.valid)
         cursor.execute('select test11')
         self.assertEqual(cursor.fetchone(), 'test11')
         cursor.execute('select test12')
@@ -357,32 +227,63 @@ class TestSteadyDB(unittest.TestCase):
         self.assertEqual(db._con.open_cursors, 0)
         self.assertEqual(db._con.num_queries, 0)
         cursor = db.cursor()
-        self.assert_(cursor.valid)
+        self.assertTrue(cursor.valid)
         cursor.callproc('test')
         cursor._cursor.valid = False
-        self.assert_(not cursor.valid)
-        self.assertRaises(InternalError, cursor._cursor.callproc, 'test')
+        self.assertFalse(cursor.valid)
+        self.assertRaises(dbapi.InternalError, cursor._cursor.callproc, 'test')
         cursor.callproc('test')
-        self.assert_(cursor.valid)
+        self.assertTrue(cursor.valid)
         cursor._cursor.callproc('test')
         self.assertEqual(db._usage, 2)
         self.assertEqual(db._con.num_uses, 3)
         db._con.valid = cursor._cursor.valid = False
         cursor.callproc('test')
-        self.assert_(cursor.valid)
+        self.assertTrue(cursor.valid)
         self.assertEqual(db._usage, 1)
         self.assertEqual(db._con.num_uses, 1)
         cursor.execute('set doit')
         db.commit()
         cursor.execute('set dont')
         db.rollback()
-        self.assertEqual(db._con.session,
-            ['doit', 'commit', 'dont', 'rollback'])
+        self.assertEqual(
+            db._con.session, ['doit', 'commit', 'dont', 'rollback'])
 
-    def test05_ConnectionCreatorFunction(self):
-        db1 = SteadyDBconnect(dbapi, 0, None, None, None, True,
+    def test05_ConnectionContextHandler(self):
+        db = SteadyDBconnect(
+            dbapi, 0, None, None, None, True,
             'SteadyDBTestDB', user='SteadyDBTestUser')
-        db2 = SteadyDBconnect(connect, 0, None, None, None, True,
+        self.assertEqual(db._con.session, [])
+        with db as con:
+            con.cursor().execute('select test')
+        self.assertEqual(db._con.session, ['commit'])
+        try:
+            with db as con:
+                con.cursor().execute('error')
+        except dbapi.ProgrammingError:
+            error = True
+        else:
+            error = False
+        self.assertTrue(error)
+        self.assertEqual(db._con.session, ['commit', 'rollback'])
+
+    def test06_CursorContextHandler(self):
+        db = SteadyDBconnect(
+            dbapi, 0, None, None, None, True,
+            'SteadyDBTestDB', user='SteadyDBTestUser')
+        self.assertEqual(db._con.open_cursors, 0)
+        with db.cursor() as cursor:
+            self.assertEqual(db._con.open_cursors, 1)
+            cursor.execute('select test')
+            self.assertEqual(cursor.fetchone(), 'test')
+        self.assertEqual(db._con.open_cursors, 0)
+
+    def test07_ConnectionCreatorFunction(self):
+        db1 = SteadyDBconnect(
+            dbapi, 0, None, None, None, True,
+            'SteadyDBTestDB', user='SteadyDBTestUser')
+        db2 = SteadyDBconnect(
+            dbapi.connect, 0, None, None, None, True,
             'SteadyDBTestDB', user='SteadyDBTestUser')
         self.assertEqual(db1.dbapi(), db2.dbapi())
         self.assertEqual(db1.threadsafety(), db2.threadsafety())
@@ -392,26 +293,30 @@ class TestSteadyDB(unittest.TestCase):
         db2.close()
         db1.close()
 
-    def test06_ConnectionMaxUsage(self):
+    def test08_ConnectionMaxUsage(self):
         db = SteadyDBconnect(dbapi, 10)
         cursor = db.cursor()
         for i in range(100):
             cursor.execute('select test%d' % i)
             r = cursor.fetchone()
             self.assertEqual(r, 'test%d' % i)
-            self.assert_(db._con.valid)
+            self.assertTrue(db._con.valid)
             j = i % 10 + 1
             self.assertEqual(db._usage, j)
             self.assertEqual(db._con.num_uses, j)
             self.assertEqual(db._con.num_queries, j)
         self.assertEqual(db._con.open_cursors, 1)
+        db.begin()
         for i in range(100):
             cursor.callproc('test')
-            self.assert_(db._con.valid)
-            j = i % 10 + 1
+            self.assertTrue(db._con.valid)
+            if i == 49:
+                db.commit()
+            j = i % 10 + 1 if i > 49 else i + 11
             self.assertEqual(db._usage, j)
             self.assertEqual(db._con.num_uses, j)
-            self.assertEqual(db._con.num_queries, 0)
+            j = 0 if i > 49 else 10
+            self.assertEqual(db._con.num_queries, j)
         for i in range(10):
             if i == 7:
                 db._con.valid = cursor._cursor.valid = False
@@ -426,10 +331,10 @@ class TestSteadyDB(unittest.TestCase):
             if i == 5:
                 db._con.valid = cursor._cursor.valid = False
             cursor.callproc('test')
-            j = (i + (i < 5 and 3 or -5)) % 10 + 1
+            j = (i + (3 if i < 5 else -5)) % 10 + 1
             self.assertEqual(db._usage, j)
             self.assertEqual(db._con.num_uses, j)
-            j = i < 5 and 3 or 0
+            j = 3 if i < 5 else 0
             self.assertEqual(db._con.num_queries, j)
         db.close()
         cursor.execute('select test1')
@@ -438,17 +343,17 @@ class TestSteadyDB(unittest.TestCase):
         self.assertEqual(db._con.num_uses, 1)
         self.assertEqual(db._con.num_queries, 1)
 
-    def test07_ConnectionSetSession(self):
+    def test09_ConnectionSetSession(self):
         db = SteadyDBconnect(dbapi, 3, ('set time zone', 'set datestyle'))
-        self.assert_(hasattr(db, '_usage'))
+        self.assertTrue(hasattr(db, '_usage'))
         self.assertEqual(db._usage, 0)
-        self.assert_(hasattr(db._con, 'open_cursors'))
+        self.assertTrue(hasattr(db._con, 'open_cursors'))
         self.assertEqual(db._con.open_cursors, 0)
-        self.assert_(hasattr(db._con, 'num_uses'))
+        self.assertTrue(hasattr(db._con, 'num_uses'))
         self.assertEqual(db._con.num_uses, 2)
-        self.assert_(hasattr(db._con, 'num_queries'))
+        self.assertTrue(hasattr(db._con, 'num_queries'))
         self.assertEqual(db._con.num_queries, 0)
-        self.assert_(hasattr(db._con, 'session'))
+        self.assertTrue(hasattr(db._con, 'session'))
         self.assertEqual(tuple(db._con.session), ('time zone', 'datestyle'))
         for i in range(11):
             db.cursor().execute('select test')
@@ -496,31 +401,31 @@ class TestSteadyDB(unittest.TestCase):
         self.assertEqual(db._con.num_queries, 1)
         self.assertEqual(db._con.session, ['time zone', 'datestyle'])
 
-    def test08_ConnectionFailures(self):
+    def test10_ConnectionFailures(self):
         db = SteadyDBconnect(dbapi)
         db.close()
         db.cursor()
-        db = SteadyDBconnect(dbapi, failures=InternalError)
+        db = SteadyDBconnect(dbapi, failures=dbapi.InternalError)
         db.close()
         db.cursor()
-        db = SteadyDBconnect(dbapi, failures=OperationalError)
+        db = SteadyDBconnect(dbapi, failures=dbapi.OperationalError)
         db.close()
-        self.assertRaises(InternalError, db.cursor)
-        db = SteadyDBconnect(dbapi,
-            failures=(OperationalError, InternalError))
+        self.assertRaises(dbapi.InternalError, db.cursor)
+        db = SteadyDBconnect(
+            dbapi, failures=(dbapi.OperationalError, dbapi.InternalError))
         db.close()
         db.cursor()
 
-    def test09_ConnectionFailureError(self):
+    def test11_ConnectionFailureError(self):
         db = SteadyDBconnect(dbapi)
         cursor = db.cursor()
         db.close()
         cursor.execute('select test')
         cursor = db.cursor()
         db.close()
-        self.assertRaises(ProgrammingError, cursor.execute, 'error')
+        self.assertRaises(dbapi.ProgrammingError, cursor.execute, 'error')
 
-    def test10_ConnectionSetSizes(self):
+    def test12_ConnectionSetSizes(self):
         db = SteadyDBconnect(dbapi)
         cursor = db.cursor()
         cursor.execute('get sizes')
@@ -545,7 +450,8 @@ class TestSteadyDB(unittest.TestCase):
         result = cursor.fetchone()
         self.assertEqual(result, ([6, 42, 7], {None: 7, 3: 15, 9: 42}))
 
-    def test11_ConnectionPingCheck(self):
+    def test13_ConnectionPingCheck(self):
+        Connection = dbapi.Connection
         Connection.has_ping = False
         Connection.num_pings = 0
         db = SteadyDBconnect(dbapi)
@@ -554,7 +460,7 @@ class TestSteadyDB(unittest.TestCase):
         db.close()
         db.cursor().execute('select test')
         self.assertEqual(Connection.num_pings, 0)
-        self.assert_(db._ping_check() is None)
+        self.assertIsNone(db._ping_check())
         self.assertEqual(Connection.num_pings, 1)
         db = SteadyDBconnect(dbapi, ping=7)
         db.cursor().execute('select test')
@@ -562,7 +468,7 @@ class TestSteadyDB(unittest.TestCase):
         db.close()
         db.cursor().execute('select test')
         self.assertEqual(Connection.num_pings, 2)
-        self.assert_(db._ping_check() is None)
+        self.assertIsNone(db._ping_check())
         self.assertEqual(Connection.num_pings, 2)
         Connection.has_ping = True
         db = SteadyDBconnect(dbapi)
@@ -571,7 +477,7 @@ class TestSteadyDB(unittest.TestCase):
         db.close()
         db.cursor().execute('select test')
         self.assertEqual(Connection.num_pings, 2)
-        self.assert_(db._ping_check())
+        self.assertTrue(db._ping_check())
         self.assertEqual(Connection.num_pings, 3)
         db = SteadyDBconnect(dbapi, ping=1)
         db.cursor().execute('select test')
@@ -579,10 +485,10 @@ class TestSteadyDB(unittest.TestCase):
         db.close()
         db.cursor().execute('select test')
         self.assertEqual(Connection.num_pings, 3)
-        self.assert_(db._ping_check())
+        self.assertTrue(db._ping_check())
         self.assertEqual(Connection.num_pings, 4)
         db.close()
-        self.assert_(db._ping_check())
+        self.assertTrue(db._ping_check())
         self.assertEqual(Connection.num_pings, 5)
         db = SteadyDBconnect(dbapi, ping=7)
         db.cursor().execute('select test')
@@ -633,7 +539,7 @@ class TestSteadyDB(unittest.TestCase):
         Connection.has_ping = False
         Connection.num_pings = 0
 
-    def test12_BeginTransaction(self):
+    def test14_BeginTransaction(self):
         db = SteadyDBconnect(dbapi, database='ok')
         cursor = db.cursor()
         cursor.close()
@@ -642,25 +548,27 @@ class TestSteadyDB(unittest.TestCase):
         db.begin()
         cursor = db.cursor()
         cursor.close()
-        self.assertRaises(InternalError, cursor.execute, 'select test12')
+        self.assertRaises(dbapi.InternalError, cursor.execute, 'select test12')
         cursor.execute('select test12')
         self.assertEqual(cursor.fetchone(), 'test12')
         db.close()
         db.begin()
-        self.assertRaises(InternalError, cursor.execute, 'select test12')
+        self.assertRaises(dbapi.InternalError, cursor.execute, 'select test12')
         cursor.execute('select test12')
         self.assertEqual(cursor.fetchone(), 'test12')
         db.begin()
-        self.assertRaises(ProgrammingError, cursor.execute, 'error')
+        self.assertRaises(dbapi.ProgrammingError, cursor.execute, 'error')
         cursor.close()
         cursor.execute('select test12')
         self.assertEqual(cursor.fetchone(), 'test12')
 
-    def test13_WithBeginExtension(self):
+    def test15_WithBeginExtension(self):
         db = SteadyDBconnect(dbapi, database='ok')
         db._con._begin_called_with = None
+
         def begin(a, b=None, c=7):
             db._con._begin_called_with = (a, b, c)
+
         db._con.begin = begin
         db.begin(42, 6)
         cursor = db.cursor()
@@ -668,7 +576,7 @@ class TestSteadyDB(unittest.TestCase):
         self.assertEqual(cursor.fetchone(), 'test13')
         self.assertEqual(db._con._begin_called_with, (42, 6, 7))
 
-    def test14_CancelTransaction(self):
+    def test16_CancelTransaction(self):
         db = SteadyDBconnect(dbapi, database='ok')
         cursor = db.cursor()
         db.begin()
@@ -678,11 +586,13 @@ class TestSteadyDB(unittest.TestCase):
         cursor.execute('select test14')
         self.assertEqual(cursor.fetchone(), 'test14')
 
-    def test15_WithCancelExtension(self):
+    def test17_WithCancelExtension(self):
         db = SteadyDBconnect(dbapi, database='ok')
         db._con._cancel_called = None
+
         def cancel():
             db._con._cancel_called = 'yes'
+
         db._con.cancel = cancel
         db.begin()
         cursor = db.cursor()
@@ -691,18 +601,62 @@ class TestSteadyDB(unittest.TestCase):
         db.cancel()
         self.assertEqual(db._con._cancel_called, 'yes')
 
-    def test16_ResetTransaction(self):
+    def test18_ResetTransaction(self):
         db = SteadyDBconnect(dbapi, database='ok')
         db.begin()
-        self.assert_(not db._con.session)
+        self.assertFalse(db._con.session)
         db.close()
-        self.assert_(not db._con.session)
+        self.assertFalse(db._con.session)
         db = SteadyDBconnect(dbapi, database='ok', closeable=False)
         db.begin()
-        self.assert_(not db._con.session)
+        self.assertFalse(db._con.session)
         db.close()
         self.assertEqual(db._con.session, ['rollback'])
 
+    def test19_CommitError(self):
+        db = SteadyDBconnect(dbapi, database='ok')
+        db.begin()
+        self.assertFalse(db._con.session)
+        self.assertTrue(db._con.valid)
+        db.commit()
+        self.assertEqual(db._con.session, ['commit'])
+        self.assertTrue(db._con.valid)
+        db.begin()
+        db._con.valid = False
+        con = db._con
+        self.assertRaises(dbapi.InternalError, db.commit)
+        self.assertFalse(db._con.session)
+        self.assertTrue(db._con.valid)
+        self.assertIsNot(con, db._con)
+        db.begin()
+        self.assertFalse(db._con.session)
+        self.assertTrue(db._con.valid)
+        db.commit()
+        self.assertEqual(db._con.session, ['commit'])
+        self.assertTrue(db._con.valid)
+
+    def test20_RollbackError(self):
+        db = SteadyDBconnect(dbapi, database='ok')
+        db.begin()
+        self.assertFalse(db._con.session)
+        self.assertTrue(db._con.valid)
+        db.rollback()
+        self.assertEqual(db._con.session, ['rollback'])
+        self.assertTrue(db._con.valid)
+        db.begin()
+        db._con.valid = False
+        con = db._con
+        self.assertRaises(dbapi.InternalError, db.rollback)
+        self.assertFalse(db._con.session)
+        self.assertTrue(db._con.valid)
+        self.assertIsNot(con, db._con)
+        db.begin()
+        self.assertFalse(db._con.session)
+        self.assertTrue(db._con.valid)
+        db.rollback()
+        self.assertEqual(db._con.session, ['rollback'])
+        self.assertTrue(db._con.valid)
+
 
 if __name__ == '__main__':
     unittest.main()
